SUBROUTINE make_profiles(GP_flag,g0,Rt,Tsurf,Psurf_v,Psurf_c,Psurf_o,Ttop,Nz, &
     Ztab,Ttab,Pv,Pc,Po,rho_v,rho_c,rho_o,i_bottom,i_top)

  implicit none

  double precision, intent(in) :: g0,Rt,Tsurf,Psurf_v,Psurf_c,Psurf_o
  integer, intent(in) :: Nz, GP_flag ! 0: water as steam ; 1: water as GP
  double precision, intent(inout) :: Ttop

  integer, intent(out) :: i_bottom, i_top
  double precision, intent(out), dimension(0:Nz) :: Ztab, Ttab, Pv, Pc, Po, rho_v, rho_c, rho_o

  integer :: profil_flag ! 0:dry, 1:moist, 2:isothermal ; 
  integer :: iz
  double precision :: a_v, b_v, k_oc, m_n, Cp_n ! alpha_v et beta_v
  double precision :: g, dTdP, dlna, dpsat

  double precision, parameter :: Rc=8.314472, M_c = 44.01e-3, M_v = 18.02e-3, M_o = 28.01e-3
  double precision, parameter :: Pmin = 0.01 ! Pa = 1e-7 bar
  double precision, dimension(0:Nz) :: lnPtab, Pn, rho_n ! Pn = Pc + Po ; rho_n = rho_c + rho_o
  double precision, dimension(1:Nz) :: dlnPtab

  double precision :: beta, psat_wrapper, get_dTdP, tsat_wrapper

  ! START

  ! protection contre temperatures trop froides
  IF (Tsurf < 273.15) STOP 'SURFACE FREEZING: ABORT'

  Ttab(0) = Tsurf
  Pv(0) = Psurf_v
  Pc(0) = Psurf_c
  Po(0) = Psurf_o
  Pn(0) = Pc(0)+Po(0)
  

  rho_c(0) = M_c*Pc(0)/(Rc*Ttab(0))
  rho_o(0) = M_o*Po(0)/(Rc*Ttab(0))
  
  rho_n(0) = rho_c(0) + rho_o(0)

  k_oc = Pc(0)/Po(0) ! fixe partout lorsque ni _c ni _o ne condensent
  m_n = rho_n(0)/(rho_c(0)/m_c+rho_o(0)/m_o) ! m_n moyen

  profil_flag = 0
  i_top = Nz-1
  i_bottom = Nz

  write(*,*) 'Psat_v (Tsurf) = ',psat_wrapper(Tsurf)
  write(*,*) 'Psurf_v = ',Psurf_v

  IF (psat_wrapper(Tsurf) < Psurf_v) THEN
     write(*,*) 'Saturation at surface: switching to wet adiabat'
     profil_flag = 1
     i_bottom = 1
     Pv(0) = psat_wrapper(Tsurf)
  END IF

  b_v = beta(Pv(0),Ttab(0),GP_flag)

  rho_v(0) = b_v*M_v*Pv(0)/(Rc*Ttab(0))


  a_v = rho_v(0)/rho_n(0)

  lnPtab(0) = log(Pc(0)+Po(0)+Pv(0))
  Ztab(0) = 0.

  set_pgrid: DO iz=1,Nz
     lnPtab(iz) = lnPtab(0) + (log(Pmin)-lnPtab(0))* &
                  ((1.*iz/Nz)**16+(1.*iz/Nz))*.5
     dlnPtab(iz) = lnPtab(iz-1) - lnPtab(iz) ! >0 !
  END DO set_pgrid

  main: DO iz=1,Nz ! main loop
!     Cp_n = (rho_c(iz-1)* &
!     (22.26+5.981d-2*Ttab(iz-1)-3.501d-5*Ttab(iz-1)**2+7.469d-9*Ttab(iz-1)**3)/m_c &
!     + rho_o(iz-1)*(1.04d3+2.97d2*exp(-(9.012d2/Ttab(iz-1)))**2))/rho_n(iz-1)

! New expression for Cp_o fitted in 300-2500 K range from Chase (1998)

     Cp_n = (rho_c(iz-1)* &
     (22.26+5.981d-2*Ttab(iz-1)-3.501d-5*Ttab(iz-1)**2+7.469d-9*Ttab(iz-1)**3)/m_c &
     + rho_o(iz-1)*(1.04d3+4.88d2*exp(-(6.881d2/Ttab(iz-1)))**2))/rho_n(iz-1)

     dTdP = &
     get_dTdP(Ttab(iz-1),rho_n(iz-1),Cp_n,m_n,a_v,profil_flag,GP_flag,dlna)

     Ttab(iz) = Ttab(iz-1) - dTdP*exp(lnPtab(iz))*dlnPtab(iz)

     IF (Ttab(iz) < Ttop) THEN
        profil_flag = 2
        write(*,*) "Isothermal at P < ",1e-5*exp(lnPtab(iz))," bar"
        write(*,*) "Isothermal at Z > ",1e-3*Ztab(iz-1)," km"
        Ttab(iz) = Ttop
        ! IF (i_bottom < Nz) THEN
        i_top = iz
        ! END IF
     END IF

     SELECT CASE(profil_flag)
     CASE (1)
        !print*, 'a_v (previous) = ',a_v
        a_v = exp(log(a_v)-dlna*dTdP*exp(lnPtab(iz))*dlnPtab(iz)/Ttab(iz))
        IF (a_v < 1e-9) THEN
          write(*,*) "too dry: alpha_v set to 1e-9"
          write(*,*) "T_top set to ",Ttab(iz)," K"
          a_v = 1e-9
          Ttop = Ttab(iz)
          profil_flag = 2
        END IF
 
!        a_v = dexp(dlog(a_v)-dlna*dTdP*dexp(lnPtab(iz))*dlnPtab(iz)/Ttab(iz))      
 
        !print*, 'a_v (next) = ',a_v

        !Pv(iz) = psat_wrapper(Ttab(iz))
        !Pn(iz) = exp(lnPtab(iz)) - Pv(iz)

        Pn(iz) = exp(lnPtab(iz))/(1.+a_v/b_v*m_n/m_v)
        Pv(iz) = exp(lnPtab(iz))/(1.+a_v/b_v*m_n/m_v)*a_v/b_v*m_n/m_v

        Ttab(iz) = tsat_wrapper(Pv(iz))

     CASE (0)
        Pn(iz) = exp(lnPtab(iz))/(1.+a_v/b_v*m_n/m_v)
        Pv(iz) = exp(lnPtab(iz))/(1.+a_v/b_v*m_n/m_v)*a_v/b_v*m_n/m_v 
        IF (Pv(iz) > psat_wrapper(Ttab(iz))) THEN
           profil_flag = 1
           i_bottom = iz
           write(*,*) "Moist at P < ",1e-5*exp(lnPtab(iz))," bar"
           write(*,*) "Moist at Z > ",1e-3*Ztab(iz-1)," km"
           !Pv(iz) = psat_wrapper(Ttab(iz))
           !Pn(iz) = exp(lnPtab(iz)) - Pv(iz)
        END IF
     CASE (2)
        Pn(iz) = exp(lnPtab(iz))/(1.+a_v/b_v*m_n/m_v)
        Pv(iz) = exp(lnPtab(iz))/(1.+a_v/b_v*m_n/m_v)*a_v/b_v*m_n/m_v 
     END SELECT

     Pc(iz) = Pn(iz)*k_oc/(1.+k_oc)
     Po(iz) =      Pn(iz)/(1.+k_oc)

     b_v = beta(Pv(iz),Ttab(iz),GP_flag)
     
     rho_n(iz) = M_n*Pn(iz)/(Rc*Ttab(iz))
     rho_c(iz) = M_c*Pc(iz)/(Rc*Ttab(iz))
     rho_o(iz) = M_o*Po(iz)/(Rc*Ttab(iz))

     ! rho_v(iz) = M_v*Pv(iz)*b_v/(Rc*Ttab(iz))
     rho_v(iz) = a_v*rho_n(iz)

     ! a_v = rho_v(iz)/rho_n(iz)

     g = g0*(Rt/(Rt+Ztab(iz-1)))**2
     Ztab(iz)= Ztab(iz-1) + exp(lnPtab(iz))/(g*(rho_v(iz)+rho_n(iz)))*dlnPtab(iz) ! hydrostat

  END DO main

  open(51,file='results/T_profile.txt')
  open(52,file='results/P_profile.txt')
  open(53,file='results/rho_profile.txt')
  open(54,file='results/alphav_profile.txt')
  open(55,file='results/rho_H2O_profile.txt')
  open(56,file='results/rho_CO2_profile.txt')


  affichage: DO iz=0,Nz
      write(51,'(ES9.3,F8.2)') Ttab(iz), Ztab(iz)*1e-3
      write(52,'(ES9.3,F8.2)') (Pv(iz)+Pc(iz)+Po(iz)), Ztab(iz)*1e-3
      write(53,'(ES9.3,F8.2)') (rho_v(iz)+rho_c(iz)+rho_o(iz)), Ztab(iz)*1e-3
      write(54,'(ES9.3,F8.2)') (rho_v(iz))/(rho_c(iz)+rho_o(iz)), Ztab(iz)*1e-3
      write(55,'(ES9.3,F8.2)') rho_v(iz), Ztab(iz)*1e-3
      write(56,'(ES9.3,F8.2)') rho_c(iz), Ztab(iz)*1e-3
   END DO affichage

   close(51)
   close(52)
   close(53)
   close(54)
   close(55)
   close(56)

END SUBROUTINE make_profiles

double precision FUNCTION beta(P_v,T,GP_flag)
  
  implicit none

  double precision, intent(in) :: P_v,T
  integer, intent(in) :: GP_flag
  double precision :: rhov, dpdr, rho_init
  double precision, parameter :: Rc = 8.314472, Mv = 18.02e-3

  IF ((GP_flag == 1).OR.(P_v < 1e4)) THEN
     !print*, 'water vapor as ideal gas'
     beta = 1.
  ELSE
     !rho_init = Mv*P_v/(Rc*T)*1d-3 ! GP en G/CM3
     rho_init = 1d-7 

     CALL dense( P_v*1d-6, T, rho_init, rhov, dpdr)
 
     beta = Rc*T*rhov*1d3/(Mv*P_v)
     IF ((beta < 0.9).OR.(beta > 1.1)) THEN
         print*, 'Significant departure from ideal gas'
         print*, 'beta = ',beta
     END IF
  END IF

  RETURN

END FUNCTION BETA

double precision FUNCTION psat_wrapper(T)

  implicit none

  double precision, intent(in) :: T
  double precision :: psat_v,rhol,rhov
  
  psat_v = 1d12 ! infinity

  IF ((T < 647.).AND.(T >= 280.)) THEN
     call psat(T,psat_v,rhol,rhov)
  ELSE IF (T < 280.) THEN
     call psat_est(T,psat_v)
  ENDIF

  psat_wrapper = psat_v * 1d6 ! -> Pa

  RETURN

END FUNCTION psat_wrapper

double precision FUNCTION tsat_wrapper(P)

  double precision, intent(in) :: P ! in Pa
  double precision :: rhol, rhov, Tsatur, lnP

  IF (P > 50d0) THEN
     CALL tsat(P*1d-6, Tsatur, rhol, rhov)
  ELSE ! low pressure : polynominal fit
     lnP = log(P)
     Tsatur = 208.0653+7.4344*lnP+0.37985*lnP**2
  END IF

  tsat_wrapper = Tsatur

  RETURN

END FUNCTION TSAT_WRAPPER



double precision FUNCTION get_dTdP(T,rhon,Cp_n,M_n,av,profil_flag,GP_flag,dlna)

  implicit none

  integer, intent(in) :: profil_flag,GP_flag
  double precision, intent(in) :: T, rhon, av, Cp_n, M_n
  double precision, intent(out) :: dlna
  
  double precision, parameter :: Rc=8.314472, M_v=18.02e-3
  double precision :: Cp_v, rho_plus, rho_minus, rhov, dVdT
  double precision :: a, cjth, cjtt, cv, dpdr, dpdr2,dpdt, gibbs, h, press, s, u

  double precision :: rmn,dlnr, dpsat, dsv,psat_minus,psat_plus,rhoc,s_c,s_minus,s_plus,s_v

  rhov = av*rhon
  dlna = 0. 

  SELECT CASE(profil_flag)
  CASE(0) ! dry adiabat
     IF ((T > 260.) .AND. (rhov > 1d-5)) THEN ! hot_adiabat
        call therm(T,rhov*1d-3,a,cjth,cjtt,Cp_v,cv,dpdr,dpdt,gibbs,h,press,s,u)
        Cp_v = 1d3 * cp_v ! kJ/kg/K -> J/kg/K

        call dense(press,T-1d-1,rhov*1d-3, rho_minus, dpdr2)
        rho_minus = rho_minus*1d3
        call dense(press,T+1d-1,rhov*1d-3, rho_plus,  dpdr2)
        rho_plus = rho_plus*1d3

        dVdT = -(rho_plus-rho_minus)/(2d-1*rhov**2)

        !get_dTdP = 1./(1.+av)*(1.+1./(T*dVdT)/rhon) / &
        !     (rhon*Cp_n + 1./(T*dVdT)*Cp_v)
        !get_dTdP = 1./(rhon*Cp_n+Cp_v/(T*dVdT))
        IF (GP_flag == 0) THEN
          get_dTdP = rhov*T*dVdT/(rhon*Cp_n+rhov*Cp_v)
        ELSE
          get_dTdP = 1./(rhon*Cp_n+rhov*Cp_v)
        ENDIF

     ELSE ! cold_adiabat
        Cp_v = (32.24+1.923d-3*T+1.055d-5*T**2-3.511d-9*T**3)/m_v
        get_dTdP = 1./(rhon*cp_n+rhov*cp_v)
     ENDIF

  CASE(1) ! wet_adiabat
     rmn = Rc/m_n

     IF (T > 273.) THEN ! hot
        CALL psat(T-0.2,psat_minus,rhoc,rho_minus)
        CALL psat(T+0.2,psat_plus ,rhoc,rho_plus )
        CALL psat(T,press,rhoc,rhov)

        psat_plus  = psat_plus * 1d6 ! MPa -> Pa
        psat_minus = psat_minus* 1d6 
        rhoc = rhoc*1d3 ! g/cm3 -> kg/m3        
        rhov = rhov*1d3
        rho_minus = rho_minus * 1d3
        rho_plus  = rho_plus  * 1d3

     ELSE ! cold
        CALL psat_est(T-0.2,psat_minus)
        CALL psat_est(T+0.2,psat_plus )

        psat_plus  = psat_plus * 1d6 ! MPa -> Pa
        psat_minus = psat_minus* 1d6 ! MPa -> Pa

        rho_minus = m_v*psat_minus/(Rc*(T-0.2))
        rho_plus  = m_v*psat_plus/(Rc*(T+0.2))
     END IF

     dpsat = (psat_plus-psat_minus)/0.4

     dlnr = T/rhov * (rho_plus-rho_minus)/0.4

     CALL therm(T-0.2,rho_minus*1d-3,a,cjth,cjtt,cp_v,cv,dpdr,dpdt,gibbs,h,press,s_minus,u)
     CALL therm(T+0.2,rho_plus *1d-3,a,cjth,cjtt,cp_v,cv,dpdr,dpdt,gibbs,h,press,s_plus ,u)

     s_plus = s_plus *1d3 ! kJ/kg/K -> J/kg/K
     s_minus= s_minus*1d3

     dsv = (s_plus-s_minus)/0.4

     IF (T > 273.) THEN ! hot
        CALL therm(T,rhoc*1d-3,a,cjth,cjtt,cp_v,cv,dpdr,dpdt,gibbs,h,press,s_c,u)
        s_c = s_c * 1d3
     ELSE ! cold
        s_c = 2278.*log(T/273.15)
     END IF

     s_v = (s_plus+s_minus)/2.
     
     dlna = (rmn*dlnr - Cp_n + rmn - av*T*dsv)/(av*(s_v-s_c)+rmn)
        
     get_dTdP = 1./(dpsat + rhon*rmn*(1.+dlnr-dlna))        
  
  CASE(2) ! isothermal
     get_dTdP = 0.
  END SELECT

  RETURN

END FUNCTION get_dTdP

