SUBROUTINE radconv(Tsurf, Ttop_in, Ttop_out, Psurf_v, Psurf_c, Psurf_o, & 
flux_up, P0_v, P0_c, P0_o, spectrum, Ttop_flag, cloud_flag, flag_cont, &
flag_grey, GP_flag, spher_flag, alb_clear, alb_cloud, const_sol, g0, Rt, Nz)

  implicit none

  integer, parameter :: Nv=36, Niter=12

  double precision, intent(in)  :: Tsurf, Psurf_c, Psurf_v, Psurf_o, Ttop_in
  double precision, intent(out) :: flux_up, P0_v, P0_c, P0_o, Ttop_out
  double precision, intent(out), dimension(Nv) :: spectrum

  double precision, intent(in) :: g0,Rt
  double precision :: Ttop, kext_top, Mmol

  integer, intent(in) :: flag_cont, flag_grey, Ttop_flag, cloud_flag
  integer, intent(in) :: GP_flag, Nz, spher_flag 
  integer :: i, it, ig, iv, i_bottom, i_top, iz 
  integer :: ng, nP, nQ, nT, nmolec

  double precision, dimension(0:Nv) :: sigmas = (/ 0., &
  40., 160., 280., 380., 500., 582., 600., 720., 752., 800., 900.,&
  1000., 1200., 1350., 1450., 1550., 1650., 1750., 1850., 1950.,&
  2050., 2200., 2500., 2800., 3200., 3600., 4000., 4400., 4800.,&
  5900.,6000., 6500., 8000., 8300., 9300., 10100. /) ! invcm 

  double precision :: sig_deb, sig_fin 
  double precision :: taueqone, Z_unit, TB, tbril

  double precision, allocatable, dimension(:) :: Ttab, Ztab, Pv, Pc, Po, P, rho_v, rho_c, rho_o
  double precision, allocatable, dimension(:) :: tau
  double precision, dimension(Nv) :: flux_up0
  double precision :: dflux_dtau0, dflux_dtau, T_b, T_a, df_a, df_b, norm0
  
  real :: tau_cloud, alb_SW
  real, intent(in) :: alb_clear, alb_cloud, const_sol

  real, allocatable :: Tarr(:), Parr(:), Qarr(:), garr(:)
  double precision, allocatable :: kdist(:,:,:,:)
  double precision, allocatable :: kdist_g(:,:,:)
  character*10, allocatable :: molec_names(:)
  character*2 :: fband

  Ttop = Ttop_in

  allocate(tau(Nz))
  allocate(Ttab(0:Nz), Ztab(0:Nz), Pv(0:Nz), Pc(0:Nz), Po(0:Nz), P(0:Nz), rho_v(0:Nz), rho_c(0:Nz), rho_o(0:Nz))

  open(9,file='kspectrum/g.dat')
  read(9,*) Ng
  allocate(garr(Ng-1))
  DO i=1,Ng-1
     read(9,*) garr(i)
  END DO
  close(9)

  open(9,file='kspectrum/T.dat')
  read(9,*) nT
  allocate(Tarr(nT))
  DO i=1,nT
     read(9,*) Tarr(i)
  END DO
  close(9)
        
  open(9,file='kspectrum/p.dat')
  read(9,*) nP
  allocate(Parr(nP))
  DO i=1,nP
     read(9,*) Parr(i)      ! log_10 P
     Parr(i) = Parr(i)+2.    ! mbar -> Pa
  END DO
  close(9)

  open(9,file='kspectrum/Q.dat')
  read(9,*) nmolec
  allocate(molec_names(nmolec))
  DO i=1,nmolec
     read(9,*) molec_names(i)
  END DO
  read(9,*) nQ
  allocate(Qarr(nQ))
  DO i=1,nQ
     read(9,*) Qarr(i)
  END DO
  close(9)           

  allocate(kdist(nT,nP,nQ,Ng))
  allocate(kdist_g(nT,nP,nQ))
  
  iter: DO it=1,Niter
     
     dflux_dtau0 = 0.
     norm0 = 0.

     CALL make_profiles(GP_flag,g0,Rt,Tsurf,Psurf_v,Psurf_c,Psurf_o,Ttop,Nz, &
          Ztab, Ttab, Pv, Pc, Po, rho_v, rho_c, rho_o, i_bottom, i_top)

     P0_c = Pc(0)
     P0_v = Pv(0)
     P0_o = Po(0)

     open(52,file='results/spectrum.txt')
     ! open(51,file='results/tau_IR.txt')

     ! estim_flux = 0d0

     bands: DO iv=1,Nv
        sig_deb = sigmas(iv-1)
        sig_fin = sigmas(iv)
        flux_up0(iv) = 0.

        IF (flag_grey == 0) THEN
           write(fband,'(I2.2)') iv-1
           open(14,file='kspectrum/corrk_'//fband//'.dat', &
                form='formatted')
           read(14,*) kdist
           close(14)

           g: DO ig=1,Ng-1
              kdist_g = kdist(:,:,:,ig)

              CALL make_tauk(Nz,Ttab,Ztab,rho_v,rho_c,rho_o,Pv,Pc,Po, &
                   nT, nP, nQ, kdist_g, Tarr, Parr, Qarr, sig_deb, sig_fin, &
                   tau, flag_cont)

              IF ((iv == Nv) .and. (ig == 8)) THEN
                DO iz=1,Nz
                   write(99,*) .5*(Ztab(iz)+Ztab(iz-1)),Ztab(iz)-Ztab(iz-1),&
                               tau(iz)
                END DO
                close(99)
             END IF

              ! print*, 'k_correle *** iv = ',iv,'***'
              CALL disort_wrapper(Nz, g0, i_bottom, i_top, tau, Ttab, &
                   Ztab, Tsurf, Ttop, rho_v, rho_c, rho_o, &
                   cloud_flag, tau_cloud, sig_deb, sig_fin, flux_up, &
                   dflux_dtau, kext_top)

              IF (spher_flag == 0) THEN
                 flux_up0(iv) = flux_up0(iv)+flux_up*garr(ig)                 
              ELSE
                 tbril = TB(sig_deb,sig_fin,flux_up)
                 Z_unit = taueqone(tbril,Ttab,Ztab,Nz)
                 print*, 'TB = ',tbril,' K'
                 print*, 'Z_unity = ',Z_unit*1e-3, ' km'
                 flux_up0(iv) = flux_up0(iv) + flux_up * garr(ig) * & 
                      (1.+Z_unit/Rt)**2
              END IF

              !dflux_dtau0 = dflux_dtau0 + dflux_dtau*garr(ig) !FAUX en non-gris
              dflux_dtau0 = dflux_dtau0 + garr(ig)*dflux_dtau*kext_top
              norm0 = norm0 + garr(ig)*flux_up*kext_top

              print*, 'k_corr, iv = ',iv,' ; g=',ig,' ; kext_top = ',kext_top
!              print*, 'dflux_dtau0 = ',dflux_dtau0
!              print*, 'norm0 = ',norm0
              print*, '***'
              
              IF (kext_top == 0) STOP "kext_top = 0 !!!"

           END DO g
        ELSE ! grey case : flag_grey <> 0

           print*, 'grey *** iv = ',iv,'***'

           CALL make_tau_grey(Nz,Ttab,Ztab,rho_v,rho_c,rho_o, &
                sig_deb,sig_fin,tau,flag_cont)

           DO iz=Nz,1,-1
              IF (tau(iz) > 50.) THEN
                 tau(iz) = 50.
              END IF
              IF (tau(iz) < 0.) THEN
                 tau(iz) = 1d-12
              END IF
           END DO

           CALL disort_wrapper(Nz, g0, i_bottom, i_top, tau, Ttab, &
                Ztab, Tsurf, Ttop, rho_v, rho_c, rho_o, &
                cloud_flag, tau_cloud, sig_deb, sig_fin, flux_up, &
                dflux_dtau, kext_top)
           IF (spher_flag == 0) THEN
              flux_up0(iv) = flux_up 
           ELSE
              tbril = TB(sig_deb,sig_fin,flux_up)
              Z_unit = taueqone(tbril,Ttab,Ztab,Nz)
              print*, 'TB = ',tbril,' K'
              print*, 'Z_unity = ',Z_unit*1e-3, ' km'
              flux_up0(iv) = flux_up * (1.+Z_unit/Rt)**2
           END IF
           
           dflux_dtau0 =  dflux_dtau0 + dflux_dtau ! OK en gris
           
        END IF ! flag_grey

        write(52,*) sig_deb, flux_up0(iv)/(sig_fin-sig_deb) ! W/m2/invcm
        write(52,*) sig_fin, flux_up0(iv)/(sig_fin-sig_deb)
        print*, 'before iv, Nz',iv,Nz
        spectrum(iv) = flux_up0(iv)/(sig_fin-sig_deb)
        print*, 'after iv, Nz',iv, Nz

     END DO bands

     flux_up = sum(flux_up0)
     IF (flag_grey == 0) THEN
        dflux_dtau0 = dflux_dtau0/norm0*flux_up
     END IF

     !close(51)
     close(52)

     alb_SW = alb_clear*exp(-tau_cloud)+alb_cloud*(1-exp(-tau_cloud))

     open(53,file='results/result.txt')
     write(53,*) 'Iteration ',it,'/',Niter
     write(53,*) '*** INPUTS ***'
     write(53,*) 'Tsurf [K] = ',Tsurf
     write(53,*) 'Ptotal_H2O [bar] = ',Psurf_v/1d5
     write(53,*) 'Patmos_H2O [bar] = ',P0_v/1d5
     write(53,*) 'Patmos_CO2 [bar] = ',P0_c/1d5
     write(53,*) 'Patmos_N2  [bar] = ',P0_o/1d5
     write(53,*) '*** RESULTS ***'
     write(53,*) 'OLR [W/m^2] = ', flux_up
     write(53,*) 'T_eff [K] = ',(flux_up/5.67e-8)**.25
     write(53,*) 'T_eq [K] = ',((1. - alb_SW)*const_sol/5.67e-8)**.25
     write(53,*) 'T_top (2 streams) [K] = ', (.5*(flux_up/5.67d-8))**.25
     write(53,*) 'Actual T_top [K] = ', Ttop
     write(53,*) 'dOLR/dtau (top) [W/m^2] = ',dflux_dtau0
     close(53)

     IF (Ttop_flag == 1) THEN
        exit iter
     ELSE IF (it == 1) THEN
        T_a = Ttop
        df_a = dflux_dtau0/flux_up
        IF (df_a < 0) THEN
           print*, 'Ttop = ',Ttop
           print*, 'initial Ttop still too high !'
           print*, 'mesosphere NOT in rad equilibrium'
           print*, 'aborting iterative Ttop adjustment'
           exit iter
        END IF
        Ttop = 2*(flux_up/5.67e-8)**.25 ! 2*Teff
     ELSE IF (it == 2) THEN
        T_b = Ttop
        df_b = dflux_dtau0/flux_up
        Ttop = (T_b*df_a - T_a*df_b)/(df_a-df_b)
        IF (df_a*df_b > 0) THEN 
           print*, 'T_a, df_a = ',T_a, df_a
           print*, 'T_b, df_b = ',T_b, df_b
           print*, "WARNING: secant fail !"
        END IF
     ELSE ! it > 2
        print*, "iter = ",it
        
        IF (df_b*dflux_dtau0 > 0) THEN
          T_b = Ttop
          df_b = dflux_dtau0/flux_up
          Ttop = (T_b*df_a - T_a*df_b)/(df_a-df_b)
        ELSE IF (df_a*dflux_dtau0 > 0) THEN
          T_a = Ttop
          df_a = dflux_dtau0/flux_up
          Ttop = (T_b*df_a - T_a*df_b)/(df_a-df_b)
        ELSE ! fallback : Ttop = Tskin (grey)
          Ttop = (.5*flux_up/5.67e-8)**.25
          print*, 'Secant fail : reverting to Tskin'
          exit iter
        END IF
        
        IF (abs(dflux_dtau0/flux_up) < 2d-2) exit iter
     END IF

  END DO iter

  flux_up = flux_up - (1. - alb_SW)*const_sol

  deallocate(tau,Ttab,Ztab,Pv,Pc,Po,P,rho_v,rho_c,rho_o)

  Ttop_out = Ttop
     
END SUBROUTINE RADCONV

double precision function taueqone(tbril, Ttab,Ztab,Nz)

implicit none

integer, intent(in) :: Nz
double precision, intent(in), dimension(0:Nz) :: Ztab
double precision, intent(in), dimension(Nz) :: Ttab
double precision, intent(in) :: tbril

integer :: iz
double precision :: alpha
logical :: found

found = .false.

find: DO iz=Nz-1,1,-1
   IF (Ttab(iz) > tbril*1.01) THEN !
      found= .true.
      exit find
   END IF
END DO find

IF (found) THEN
  alpha = (Ttab(iz)-tbril)/(Ttab(iz)-Ttab(iz+1))
  taueqone = Ztab(iz+1)*alpha+(1.-alpha)*Ztab(iz)
ELSE
  taueqone = 0.
END IF

return

END function TAUEQONE

double precision function TB(sig_min,sig_max,flux)

implicit none

double precision, intent(in) :: sig_min, sig_max, flux
double precision :: sig_mid, dsig

!sig_mid = (sig_min*sig_max)**.5
sig_mid = .5*(sig_min+sig_max)
dsig = (sig_max - sig_min)

TB = 1.43878*sig_mid/(dlog(dsig*sig_mid**3/flux + 2.67253e7)-17.1011)

return

END function TB
